#include <iostream.h>
#include <stdlib.h>
#include "CPstring.h"
#include "rando.h"
#include "vector.h"
#include "prompt.h"

#include "bet.h"
#include "wheel.h"
#include "bankroll.h"


// create different bet classes based on
// abstract class Bet

// handle odd/even bets

class OddEvenBet : public Bet
{
 public:
    OddEvenBet(Wheel & w);
    virtual string description() const;
    virtual int deltaMoney()     const;
    virtual void placeBet(const BankRoll & br);
 private:
    bool myIsEven;
};

OddEvenBet::OddEvenBet(Wheel & w)
    : Bet(w),
      myIsEven(true)
{
    
}

string OddEvenBet::description() const
{
    return "bet on odd/even number, payoff 1:1";
}

int OddEvenBet::deltaMoney() const
{
    if (myWheel.getColor() != Wheel::green &&
	(myWheel.getNumber() % 2 == 0) == myIsEven)
    {
	return myBetAmount;
    }
    else
    {
	return -1 * myBetAmount;
    }
}

void OddEvenBet::placeBet(const BankRoll & br)
{
    char ch;
    cout << "bet on odd or even (o/e) : ";
    cin >> ch;
    myIsEven = tolower(ch) == 'e';
    Bet::placeBet(br);
}

// handle bet of any three #'s in sequence
// (not quite how real roulette works, but easier
// to code)

class ThreeRunBet : public Bet
{
  public:
    ThreeRunBet(Wheel & w);
    virtual string description() const;
    virtual int deltaMoney()     const;
    virtual void placeBet(const BankRoll & br);

  private:
    
    int myNumber;
};

ThreeRunBet::ThreeRunBet(Wheel & w)
    : Bet(w),
      myNumber(0)
{
    
}

string ThreeRunBet::description() const
{
    return "bet on three consecutive numbers";
}

void ThreeRunBet::placeBet(const BankRoll & br)
{
    cout << "enter first of three consecutive numbers" << endl;
    cout << "i.e., 1 for (1, 2, 3) and 34 for (34, 35, 36) : ";
    myNumber = PromptRange("",1,34);
    Bet::placeBet(br);
}

int ThreeRunBet::deltaMoney() const
{
    int num = myWheel.getNumber();
    if (myNumber <= num && num <= myNumber+2)
    {
	return 11 * myBetAmount;
    }
    else
    {
	return -1 *myBetAmount;
    }
}


// handle red/black bet

class BlackRedBet : public Bet
{
 public:
    BlackRedBet(Wheel & w);
    virtual string description() const;
    virtual int deltaMoney()     const;
    virtual void placeBet(const BankRoll & br);

 private:
    Wheel::Color myColor;
};


BlackRedBet::BlackRedBet(Wheel & w)
    : Bet(w),
      myColor(Wheel::red)
{

}

string BlackRedBet::description() const
{
    return "bet on black or red, payoff 1:1";
}

void BlackRedBet::placeBet(const BankRoll & br)
{

    char ch;
    cout << "bet on red or black (r/b) : ";
    cin >> ch;
    myColor = tolower(ch) == 'r' ? Wheel::red : Wheel::black;
    Bet::placeBet(br);
}

int BlackRedBet::deltaMoney() const
{
    if (myWheel.getColor() == myColor)
    {
	return myBetAmount;
    }
    else
    {
	return -1 * myBetAmount;
    }
}

class Game
{
  public:
    
    Game();
    
    bool Continue();      // returns true if game continues to play
    void MakeBet();       // place a wager for one turn of the game
    void Play();          // play the game once

    void AddBet(Bet * b);
    
  private:
    
    Wheel myWheel;              // use this wheel
    BankRoll myMoney;           // bankroll (only one player)
    Vector<Bet *> myBets;       // all the different bets
    int myNumBets;              // # of bets
    Bet * myBetMade;            // which bet is actually made
    bool myContinue;            // should we continue?
};


Game::Game()
    : myWheel(),
      myMoney(1000),
      myBets(10),
      myNumBets(0),
      myBetMade(0),
      myContinue(true)
{
    AddBet(new BlackRedBet(myWheel));
    AddBet(new OddEvenBet(myWheel));
    AddBet(new ThreeRunBet(myWheel));
}

void Game::AddBet(Bet * b)
{
    if(myNumBets < myBets.length())   // if there's room for the bet
    {
	myBets[myNumBets] = b;
	myNumBets++;
    }
}

bool Game::Continue()
{
    return myMoney.isSolvent() && myContinue;
}

void Game::MakeBet()
{
    int k;
    int index;
    for(k=0; k < myNumBets; k++)
    {
	cout << k << ") " << myBets[k]->description() << endl;
    }
    cout << k << ") QUIT" << endl;
    index = PromptRange("choice: ",0,myNumBets);
    if (index == myNumBets)
    {
	myContinue = false;
	return;
    }
    myBetMade = myBets[index];
    myBetMade->placeBet(myMoney);
    cout << endl;
}

void Game::Play()
{
    MakeBet();
    if (! Continue())
    {
	return;
    }
    int k;
    for(k=0; k < 5; k++)
    {
	myWheel.spin();	
    }
    cout << endl << endl;

    myMoney.processBet(myBetMade);
    myMoney.report();
}

int main(int argc, char * argv[])
{    
    Game g;
    do
    {
	g.Play();
    } while (g.Continue());
    return 0;
}

